<?php

/**
 * @file
 * Features Export Explode explosion code.
 */

/**
 * Explode the Features.
 */
function fxx_export_explode() {
  $features = features_get_features();

  foreach ($features as $feature) {
    foreach ($feature->components as $type) {
      fxx_export_explode_feature_components($feature->name, $type);
    }
  }
}

/**
 * Explodes a component type for a Feature module.
 *
 * @param string $feature
 * @param string $type
 */
function fxx_export_explode_feature_components($feature, $type) {
  $fxx_info = fxx_info();

  if (empty($fxx_info[$type])) {
    return;
  }

  $info = features_get_components();

  if ($components = fxx_load_export($feature, $type, $info)) {
    $files = fxx_write_exploded_files($feature, $type, $components);
    fxx_rewrite_default_hook($feature, $type, $info, $files);
  }
}

/**
 * Find the "default" include file containing the component code.
 *
 * @param string $feature
 *   Feature module name.
 * @param string $type
 *   Component type.
 * @param array $info
 *   Entire Feature modules info array, obtained from features_get_components().
 *
 * @return string
 *   The file name of the file, NULL if unable to determine.
 */
function fxx_get_default_file($feature, $type, $info) {
  switch ($info[$type]['default_file']) {
    case FEATURES_DEFAULTS_INCLUDED:
      $file = "$feature.features.$type";
      break;
    case FEATURES_DEFAULTS_CUSTOM:
      if (!empty($info[$type]['default_filename'])) {
        $file = "$feature." . $info[$type]['default_filename'];
      } else {
        $file = FALSE;
      }
      break;
    default:
      $file = "$feature.features";
  }

  if ($file) {
    return $file;
  }
}

/**
 * Load the "default" include file containing the component code.
 *
 * @param string $feature
 *   Feature module name.
 * @param string $type
 *   Component type.
 * @param array $info
 *   Entire Feature modules info array, obtained from features_get_components().
 */
function fxx_load_default_file($feature, $type, $info) {
  if ($file = fxx_get_default_file($feature, $type, $info)) {
    module_load_include('inc', $feature, $file);
  }
}

/**
 * Load the exported components from the default hook.
 *
 * @param string $feature
 *   Feature module name.
 * @param string $type
 *   Component type.
 * @param array $info
 *   Entire Feature modules info array, obtained from features_get_components().
 *
 * @return array
 *   An array of exported components.
 */
function fxx_load_export($feature, $type, $info) {
  fxx_load_default_file($feature, $type, $info);
  $default_hook = $feature . '_' . $info[$type]['default_hook'];

  if (function_exists($default_hook)) {
    $components = $default_hook();
  } else {
    $components = NULL;
  }

  return $components;
}

/**
 * Rewrites the default hook which contained the export.
 *
 * @param string $feature
 *   Feature module name.
 * @param string $type
 *   Component type.
 * @param array $info
 *   Entire Feature modules info array, obtained from features_get_components().
 * @param array $files
 *   Names of the exploded export files.
 */
function fxx_rewrite_default_hook($feature, $type, $info, $files) {
  $path = drupal_get_path('module', $feature);
  $file = fxx_get_default_file($feature, $type, $info) . '.inc';
  $contents = file($path . '/' . $file);
  $rewrite = array();

  $default_hook = $feature . '_' . $info[$type]['default_hook'];
  $pattern = '/function ' . $default_hook . '\(/';
  $state = 0;

  $fxx_info = fxx_info($type);

  foreach ($contents as $line) {
    $line = trim($line);
    switch ($state) {
      case 0:
        // Haven't yet found start of function.
        preg_match($pattern, $line, $matches);
        if ($matches) {
          $state = 1;
        }
        $rewrite[] = $line;
        break;
      case 1:
        // In the function.
        if ($line == '}') {
          $rewrite[] = "  {$fxx_info['export_var']} = array();";
          foreach ($files as $include) {
            $rewrite[] = "  include __DIR__ . '/{$include}';";
          }
          $rewrite[] = "  return {$fxx_info['export_var']};";
          $rewrite[] = $line;
        }
        break;
      default:
        // Past the function.
        $rewrite[] = $line;
        break;
    }
  }

  // End with empty line.
  $rewrite[] = '';

  file_unmanaged_save_data(implode("\n", $rewrite), $path . '/' . $file, FILE_EXISTS_REPLACE);
}

/**
 * Write the exploded export files.
 *
 * @param string $feature
 *   Feature module name.
 * @param string $type
 *   Component type.
 * @param array $components
 *   An array of the instantiated component values for export.
 *
 * @return array
 *   An array of exploded export file names.
 */
function fxx_write_exploded_files($feature, $type, $components) {
  module_load_include('inc', 'features', 'features.export');
  $path = drupal_get_path('module', $feature);
  $files = array();

  $fxx_info = fxx_info($type);

  foreach ($components as $name => $component) {
    $filename = "$feature.$type.$name.inc";
    $files[] = $filename;
    $code = "<?php\n\n";

    switch ($fxx_info['export_type']) {
      case FXX_EXPORT_TYPE_VAR:
        $code .= features_var_export($component) . "\n";
        $code .= "{$fxx_info['export_var']}['$name'] = {$fxx_info['component_var']};\n";
        break;
      case FXX_EXPORT_TYPE_VAL:
        $code .= "{$fxx_info['export_var']}['$name'] = " . features_var_export($component) . ";\n";
        break;
    }

    file_unmanaged_save_data($code, "$path/$filename", FILE_EXISTS_REPLACE);
  }

  return $files;
}

/**
 * Gather FXX integration from enabled modules.
 *
 * @param string $type
 *   Component type, optional.
 *
 * @return array
 *   FXX info array for the specified type. All types if no type specified.
 */
function fxx_info($type = NULL) {
  $info = drupal_static(__FUNCTION__, NULL);

  if ($info === NULL) {
    $info = module_invoke_all('fxx_info');
    drupal_alter(__FUNCTION__, $identifiers);
  }

  if ($type) {
    return !empty($info[$type]) ? $info[$type] : NULL;
  }

  return $info;
}
